@@ -1,5 +1,5 @@
 """Distributed Task Queue."""
-# :copyright: (c) 2016-20206 Asif Saif Uddin, celery core and individual
+# :copyright: (c) 2016-2026 Asif Saif Uddin, celery core and individual
 #                 contributors, All rights reserved.
 # :copyright: (c) 2015-2016 Ask Solem.  All rights reserved.
 # :copyright: (c) 2012-2014 GoPivotal, Inc., All rights reserved.
@@ -15,9 +15,9 @@ from collections import namedtuple
 # Lazy loading
 from . import local  # noqa
 
-SERIES = 'singularity'
+SERIES = 'sun-harmonics'
 
-__version__ = '5.0.5'
+__version__ = '5.1.1'
 __author__ = 'Ask Solem'
 __contact__ = 'auvipy@gmail.com'
 __homepage__ = 'http://celeryproject.org'
@@ -142,7 +142,8 @@ def maybe_patch_concurrency(argv=None, short_opts=None,
 
         # set up eventlet/gevent environments ASAP
         from celery import concurrency
-        concurrency.get_implementation(pool)
+        if pool in concurrency.get_available_pool_names():
+            concurrency.get_implementation(pool)
 
 
 # this just creates a new module, that imports stuff on first attribute
