@@ -22,6 +22,7 @@ from kombu.utils.url import maybe_sanitize_url
 import celery.exceptions
 from celery import current_app, group, maybe_signature, states
 from celery._state import get_current_task
+from celery.app.task import Context
 from celery.exceptions import (BackendGetMetaError, BackendStoreError,
                                ChordError, ImproperlyConfigured,
                                NotRegistered, TaskRevokedError, TimeoutError)
@@ -76,6 +77,12 @@ class _nulldict(dict):
     __setitem__ = update = setdefault = ignore
 
 
+def _is_request_ignore_result(request):
+    if request is None:
+        return False
+    return request.ignore_result
+
+
 class Backend:
     READY_STATES = states.READY_STATES
     UNREADY_STATES = states.UNREADY_STATES
@@ -150,7 +157,7 @@ class Backend:
     def mark_as_done(self, task_id, result,
                      request=None, store_result=True, state=states.SUCCESS):
         """Mark task as successfully executed."""
-        if store_result:
+        if (store_result and not _is_request_ignore_result(request)):
             self.store_result(task_id, result, state, request=request)
         if request and request.chord:
             self.on_chord_part_return(request, state, result)
@@ -164,8 +171,44 @@ class Backend:
             self.store_result(task_id, exc, state,
                               traceback=traceback, request=request)
         if request:
+            # This task may be part of a chord
             if request.chord:
                 self.on_chord_part_return(request, state, exc)
+            # It might also have chained tasks which need to be propagated to,
+            # this is most likely to be exclusive with being a direct part of a
+            # chord but we'll handle both cases separately.
+            #
+            # The `chain_data` try block here is a bit tortured since we might
+            # have non-iterable objects here in tests and it's easier this way.
+            try:
+                chain_data = iter(request.chain)
+            except (AttributeError, TypeError):
+                chain_data = tuple()
+            for chain_elem in chain_data:
+                chain_elem_opts = chain_elem['options']
+                # If the state should be propagated, we'll do so for all
+                # elements of the chain. This is only truly important so
+                # that the last chain element which controls completion of
+                # the chain itself is marked as completed to avoid stalls.
+                if self.store_result and state in states.PROPAGATE_STATES:
+                    try:
+                        chained_task_id = chain_elem_opts['task_id']
+                    except KeyError:
+                        pass
+                    else:
+                        self.store_result(
+                            chained_task_id, exc, state,
+                            traceback=traceback, request=chain_elem
+                        )
+                # If the chain element is a member of a chord, we also need
+                # to call `on_chord_part_return()` as well to avoid stalls.
+                if 'chord' in chain_elem_opts:
+                    failed_ctx = Context(chain_elem)
+                    failed_ctx.update(failed_ctx.options)
+                    failed_ctx.id = failed_ctx.options['task_id']
+                    failed_ctx.group = failed_ctx.options['group_id']
+                    self.on_chord_part_return(failed_ctx, state, exc)
+            # And finally we'll fire any errbacks
             if call_errbacks and request.errbacks:
                 self._call_task_errbacks(request, exc, traceback)
 
@@ -590,6 +633,9 @@ class Backend:
     def on_chord_part_return(self, request, state, result, **kwargs):
         pass
 
+    def set_chord_size(self, group_id, chord_size):
+        pass
+
     def fallback_chord_unlock(self, header_result, body, countdown=1,
                               **kwargs):
         kwargs['result'] = [r.as_tuple() for r in header_result]
@@ -605,8 +651,9 @@ class Backend:
     def ensure_chords_allowed(self):
         pass
 
-    def apply_chord(self, header_result, body, **kwargs):
+    def apply_chord(self, header_result_args, body, **kwargs):
         self.ensure_chords_allowed()
+        header_result = self.app.GroupResult(*header_result_args)
         self.fallback_chord_unlock(header_result, body, **kwargs)
 
     def current_task_children(self, request=None):
@@ -853,7 +900,11 @@ class BaseKeyValueStoreBackend(Backend):
         if current_meta['status'] == states.SUCCESS:
             return result
 
-        self._set_with_state(self.get_key_for_task(task_id), self.encode(meta), state)
+        try:
+            self._set_with_state(self.get_key_for_task(task_id), self.encode(meta), state)
+        except BackendStoreError as ex:
+            raise BackendStoreError(str(ex), state=state, task_id=task_id) from ex
+
         return result
 
     def _save_group(self, group_id, result):
@@ -883,8 +934,9 @@ class BaseKeyValueStoreBackend(Backend):
             meta['result'] = result_from_tuple(result, self.app)
             return meta
 
-    def _apply_chord_incr(self, header_result, body, **kwargs):
+    def _apply_chord_incr(self, header_result_args, body, **kwargs):
         self.ensure_chords_allowed()
+        header_result = self.app.GroupResult(*header_result_args)
         header_result.save(backend=self)
 
     def on_chord_part_return(self, request, state, result, **kwargs):
@@ -928,7 +980,9 @@ class BaseKeyValueStoreBackend(Backend):
             j = deps.join_native if deps.supports_native_join else deps.join
             try:
                 with allow_join_result():
-                    ret = j(timeout=3.0, propagate=True)
+                    ret = j(
+                        timeout=app.conf.result_chord_join_timeout,
+                        propagate=True)
             except Exception as exc:  # pylint: disable=broad-except
                 try:
                     culprit = next(deps._failed_join_report())
