@@ -7,14 +7,15 @@ from urllib.parse import unquote
 
 from kombu.utils.functional import retry_over_time
 from kombu.utils.objects import cached_property
-from kombu.utils.url import _parse_url
+from kombu.utils.url import _parse_url, maybe_sanitize_url
 
 from celery import states
 from celery._state import task_join_will_block
 from celery.canvas import maybe_signature
-from celery.exceptions import ChordError, ImproperlyConfigured
+from celery.exceptions import (BackendStoreError, ChordError,
+                               ImproperlyConfigured)
 from celery.result import GroupResult, allow_join_result
-from celery.utils.functional import dictfilter
+from celery.utils.functional import _regen, dictfilter
 from celery.utils.log import get_logger
 from celery.utils.time import humanize_seconds
 
@@ -47,13 +48,13 @@ sentinel in order to use the Redis result store backend.
 
 W_REDIS_SSL_CERT_OPTIONAL = """
 Setting ssl_cert_reqs=CERT_OPTIONAL when connecting to redis means that \
-celery might not valdate the identity of the redis broker when connecting. \
+celery might not validate the identity of the redis broker when connecting. \
 This leaves you vulnerable to man in the middle attacks.
 """
 
 W_REDIS_SSL_CERT_NONE = """
 Setting ssl_cert_reqs=CERT_NONE when connecting to redis means that celery \
-will not valdate the identity of the redis broker when connecting. This \
+will not validate the identity of the redis broker when connecting. This \
 leaves you vulnerable to man in the middle attacks.
 """
 
@@ -185,6 +186,7 @@ class RedisBackend(BaseKeyValueStoreBackend, AsyncBackendMixin):
 
     #: :pypi:`redis` client module.
     redis = redis
+    connection_class_ssl = redis.SSLConnection if redis else None
 
     #: Maximum number of connections in the pool.
     max_connections = None
@@ -192,6 +194,10 @@ class RedisBackend(BaseKeyValueStoreBackend, AsyncBackendMixin):
     supports_autoexpire = True
     supports_native_join = True
 
+    #: Maximal length of string value in Redis.
+    #: 512 MB - https://redis.io/topics/data-types
+    _MAX_STR_VALUE_SIZE = 536870912
+
     def __init__(self, host=None, port=None, db=None, password=None,
                  max_connections=None, url=None,
                  connection_pool=None, **kwargs):
@@ -213,6 +219,7 @@ class RedisBackend(BaseKeyValueStoreBackend, AsyncBackendMixin):
         socket_connect_timeout = _get('redis_socket_connect_timeout')
         retry_on_timeout = _get('redis_retry_on_timeout')
         socket_keepalive = _get('redis_socket_keepalive')
+        health_check_interval = _get('redis_backend_health_check_interval')
 
         self.connparams = {
             'host': _get('redis_host') or 'localhost',
@@ -226,6 +233,20 @@ class RedisBackend(BaseKeyValueStoreBackend, AsyncBackendMixin):
                 socket_connect_timeout and float(socket_connect_timeout),
         }
 
+        username = _get('redis_username')
+        if username:
+            # We're extra careful to avoid including this configuration value
+            # if it wasn't specified since older versions of py-redis
+            # don't support specifying a username.
+            # Only Redis>6.0 supports username/password authentication.
+
+            # TODO: Include this in connparams' definition once we drop
+            #       support for py-redis<3.4.0.
+            self.connparams['username'] = username
+
+        if health_check_interval:
+            self.connparams["health_check_interval"] = health_check_interval
+
         # absent in redis.connection.UnixDomainSocketConnection
         if socket_keepalive:
             self.connparams['socket_keepalive'] = socket_keepalive
@@ -236,7 +257,7 @@ class RedisBackend(BaseKeyValueStoreBackend, AsyncBackendMixin):
         ssl = _get('redis_backend_use_ssl')
         if ssl:
             self.connparams.update(ssl)
-            self.connparams['connection_class'] = redis.SSLConnection
+            self.connparams['connection_class'] = self.connection_class_ssl
 
         if url:
             self.connparams = self._params_from_url(url, self.connparams)
@@ -245,7 +266,7 @@ class RedisBackend(BaseKeyValueStoreBackend, AsyncBackendMixin):
         # redis_backend_use_ssl dict, check ssl_cert_reqs is valid. If set
         # via query string ssl_cert_reqs will be a string so convert it here
         if ('connection_class' in self.connparams and
-                self.connparams['connection_class'] is redis.SSLConnection):
+                issubclass(self.connparams['connection_class'], redis.SSLConnection)):
             ssl_cert_reqs_missing = 'MISSING'
             ssl_string_to_constant = {'CERT_REQUIRED': CERT_REQUIRED,
                                       'CERT_OPTIONAL': CERT_OPTIONAL,
@@ -275,11 +296,11 @@ class RedisBackend(BaseKeyValueStoreBackend, AsyncBackendMixin):
         )
 
     def _params_from_url(self, url, defaults):
-        scheme, host, port, _, password, path, query = _parse_url(url)
+        scheme, host, port, username, password, path, query = _parse_url(url)
         connparams = dict(
             defaults, **dictfilter({
-                'host': host, 'port': port, 'password': password,
-                'db': query.pop('virtual_host', None)})
+                'host': host, 'port': port, 'username': username,
+                'password': password, 'db': query.pop('virtual_host', None)})
         )
 
         if scheme == 'socket':
@@ -364,6 +385,9 @@ class RedisBackend(BaseKeyValueStoreBackend, AsyncBackendMixin):
         return tts
 
     def set(self, key, value, **retry_policy):
+        if isinstance(value, str) and len(value) > self._MAX_STR_VALUE_SIZE:
+            raise BackendStoreError('value too large for Redis backend')
+
         return self.ensure(self._set, (key, value), **retry_policy)
 
     def _set(self, key, value):
@@ -401,15 +425,20 @@ class RedisBackend(BaseKeyValueStoreBackend, AsyncBackendMixin):
             raise ChordError(f'Dependency {tid} raised {retval!r}')
         return retval
 
-    def apply_chord(self, header_result, body, **kwargs):
+    def set_chord_size(self, group_id, chord_size):
+        self.set(self.get_key_for_group(group_id, '.s'), chord_size)
+
+    def apply_chord(self, header_result_args, body, **kwargs):
         # If any of the child results of this chord are complex (ie. group
         # results themselves), we need to save `header_result` to ensure that
         # the expected structure is retained when we finish the chord and pass
         # the results onward to the body in `on_chord_part_return()`. We don't
         # do this is all cases to retain an optimisation in the common case
         # where a chord header is comprised of simple result objects.
-        if any(isinstance(nr, GroupResult) for nr in header_result.results):
-            header_result.save(backend=self)
+        if not isinstance(header_result_args[1], _regen):
+            header_result = self.app.GroupResult(*header_result_args)
+            if any(isinstance(nr, GroupResult) for nr in header_result.results):
+                header_result.save(backend=self)
 
     @cached_property
     def _chord_zset(self):
@@ -431,6 +460,7 @@ class RedisBackend(BaseKeyValueStoreBackend, AsyncBackendMixin):
         client = self.client
         jkey = self.get_key_for_group(gid, '.j')
         tkey = self.get_key_for_group(gid, '.t')
+        skey = self.get_key_for_group(gid, '.s')
         result = self.encode_result(result, state)
         encoded = self.encode([1, tid, state, result])
         with client.pipeline() as pipe:
@@ -438,74 +468,80 @@ class RedisBackend(BaseKeyValueStoreBackend, AsyncBackendMixin):
                 pipe.zadd(jkey, {encoded: group_index}).zcount(jkey, "-inf", "+inf")
                 if self._chord_zset
                 else pipe.rpush(jkey, encoded).llen(jkey)
-            ).get(tkey)
+            ).get(tkey).get(skey)
             if self.expires:
                 pipeline = pipeline \
                     .expire(jkey, self.expires) \
-                    .expire(tkey, self.expires)
+                    .expire(tkey, self.expires) \
+                    .expire(skey, self.expires)
 
-            _, readycount, totaldiff = pipeline.execute()[:3]
+            _, readycount, totaldiff, chord_size_bytes = pipeline.execute()[:4]
 
         totaldiff = int(totaldiff or 0)
 
-        try:
-            callback = maybe_signature(request.chord, app=app)
-            total = callback['chord_size'] + totaldiff
-            if readycount == total:
-                header_result = GroupResult.restore(gid)
-                if header_result is not None:
-                    # If we manage to restore a `GroupResult`, then it must
-                    # have been complex and saved by `apply_chord()` earlier.
-                    #
-                    # Before we can join the `GroupResult`, it needs to be
-                    # manually marked as ready to avoid blocking
-                    header_result.on_ready()
-                    # We'll `join()` it to get the results and ensure they are
-                    # structured as intended rather than the flattened version
-                    # we'd construct without any other information.
-                    join_func = (
-                        header_result.join_native
-                        if header_result.supports_native_join
-                        else header_result.join
-                    )
-                    with allow_join_result():
-                        resl = join_func(timeout=3.0, propagate=True)
-                else:
-                    # Otherwise simply extract and decode the results we
-                    # stashed along the way, which should be faster for large
-                    # numbers of simple results in the chord header.
-                    decode, unpack = self.decode, self._unpack_chord_result
-                    with client.pipeline() as pipe:
-                        if self._chord_zset:
-                            pipeline = pipe.zrange(jkey, 0, -1)
-                        else:
-                            pipeline = pipe.lrange(jkey, 0, total)
-                        resl, = pipeline.execute()
-                    resl = [unpack(tup, decode) for tup in resl]
-                try:
-                    callback.delay(resl)
-                except Exception as exc:  # pylint: disable=broad-except
-                    logger.exception(
-                        'Chord callback for %r raised: %r', request.group, exc)
-                    return self.chord_error_from_stack(
-                        callback,
-                        ChordError(f'Callback error: {exc!r}'),
-                    )
-                finally:
-                    with client.pipeline() as pipe:
-                        _, _ = pipe \
-                            .delete(jkey) \
-                            .delete(tkey) \
-                            .execute()
-        except ChordError as exc:
-            logger.exception('Chord %r raised: %r', request.group, exc)
-            return self.chord_error_from_stack(callback, exc)
-        except Exception as exc:  # pylint: disable=broad-except
-            logger.exception('Chord %r raised: %r', request.group, exc)
-            return self.chord_error_from_stack(
-                callback,
-                ChordError(f'Join error: {exc!r}'),
-            )
+        if chord_size_bytes:
+            try:
+                callback = maybe_signature(request.chord, app=app)
+                total = int(chord_size_bytes) + totaldiff
+                if readycount == total:
+                    header_result = GroupResult.restore(gid)
+                    if header_result is not None:
+                        # If we manage to restore a `GroupResult`, then it must
+                        # have been complex and saved by `apply_chord()` earlier.
+                        #
+                        # Before we can join the `GroupResult`, it needs to be
+                        # manually marked as ready to avoid blocking
+                        header_result.on_ready()
+                        # We'll `join()` it to get the results and ensure they are
+                        # structured as intended rather than the flattened version
+                        # we'd construct without any other information.
+                        join_func = (
+                            header_result.join_native
+                            if header_result.supports_native_join
+                            else header_result.join
+                        )
+                        with allow_join_result():
+                            resl = join_func(
+                                timeout=app.conf.result_chord_join_timeout,
+                                propagate=True
+                            )
+                    else:
+                        # Otherwise simply extract and decode the results we
+                        # stashed along the way, which should be faster for large
+                        # numbers of simple results in the chord header.
+                        decode, unpack = self.decode, self._unpack_chord_result
+                        with client.pipeline() as pipe:
+                            if self._chord_zset:
+                                pipeline = pipe.zrange(jkey, 0, -1)
+                            else:
+                                pipeline = pipe.lrange(jkey, 0, total)
+                            resl, = pipeline.execute()
+                        resl = [unpack(tup, decode) for tup in resl]
+                    try:
+                        callback.delay(resl)
+                    except Exception as exc:  # pylint: disable=broad-except
+                        logger.exception(
+                            'Chord callback for %r raised: %r', request.group, exc)
+                        return self.chord_error_from_stack(
+                            callback,
+                            ChordError(f'Callback error: {exc!r}'),
+                        )
+                    finally:
+                        with client.pipeline() as pipe:
+                            pipe \
+                                .delete(jkey) \
+                                .delete(tkey) \
+                                .delete(skey) \
+                                .execute()
+            except ChordError as exc:
+                logger.exception('Chord %r raised: %r', request.group, exc)
+                return self.chord_error_from_stack(callback, exc)
+            except Exception as exc:  # pylint: disable=broad-except
+                logger.exception('Chord %r raised: %r', request.group, exc)
+                return self.chord_error_from_stack(
+                    callback,
+                    ChordError(f'Join error: {exc!r}'),
+                )
 
     def _create_client(self, **params):
         return self._get_client()(
@@ -535,10 +571,25 @@ class RedisBackend(BaseKeyValueStoreBackend, AsyncBackendMixin):
         )
 
 
+if getattr(redis, "sentinel", None):
+    class SentinelManagedSSLConnection(
+            redis.sentinel.SentinelManagedConnection,
+            redis.SSLConnection):
+        """Connect to a Redis server using Sentinel + TLS.
+
+        Use Sentinel to identify which Redis server is the current master
+        to connect to and when connecting to the Master server, use an
+        SSL Connection.
+        """
+
+
 class SentinelBackend(RedisBackend):
     """Redis sentinel task result store."""
+    # URL looks like `sentinel://0.0.0.0:26347/3;sentinel://0.0.0.0:26348/3`
+    _SERVER_URI_SEPARATOR = ";"
 
     sentinel = getattr(redis, "sentinel", None)
+    connection_class_ssl = SentinelManagedSSLConnection if sentinel else None
 
     def __init__(self, *args, **kwargs):
         if self.sentinel is None:
@@ -546,9 +597,30 @@ class SentinelBackend(RedisBackend):
 
         super().__init__(*args, **kwargs)
 
+    def as_uri(self, include_password=False):
+        """
+        Return the server addresses as URIs, sanitizing the password or not.
+        """
+        # Allow superclass to do work if we don't need to force sanitization
+        if include_password:
+            return super().as_uri(
+                include_password=include_password,
+            )
+        # Otherwise we need to ensure that all components get sanitized rather
+        # by passing them one by one to the `kombu` helper
+        uri_chunks = (
+            maybe_sanitize_url(chunk)
+            for chunk in (self.url or "").split(self._SERVER_URI_SEPARATOR)
+        )
+        # Similar to the superclass, strip the trailing slash from URIs with
+        # all components empty other than the scheme
+        return self._SERVER_URI_SEPARATOR.join(
+            uri[:-1] if uri.endswith(":///") else uri
+            for uri in uri_chunks
+        )
+
     def _params_from_url(self, url, defaults):
-        # URL looks like sentinel://0.0.0.0:26347/3;sentinel://0.0.0.0:26348/3.
-        chunks = url.split(";")
+        chunks = url.split(self._SERVER_URI_SEPARATOR)
         connparams = dict(defaults, hosts=[])
         for chunk in chunks:
             data = super()._params_from_url(
