@@ -1,6 +1,7 @@
 """Click customizations for Celery."""
 import json
 from collections import OrderedDict
+from functools import update_wrapper
 from pprint import pformat
 
 import click
@@ -8,6 +9,7 @@ from click import ParamType
 from kombu.utils.objects import cached_property
 
 from celery._state import get_current_app
+from celery.signals import user_preload_options
 from celery.utils import text
 from celery.utils.log import mlevel
 from celery.utils.time import maybe_iso8601
@@ -113,6 +115,25 @@ class CLIContext:
             self.echo(body)
 
 
+def handle_preload_options(f):
+    def caller(ctx, *args, **kwargs):
+        app = ctx.obj.app
+
+        preload_options = [o.name for o in app.user_options.get('preload', [])]
+
+        if preload_options:
+            user_options = {
+                preload_option: kwargs[preload_option]
+                for preload_option in preload_options
+            }
+
+            user_preload_options.send(sender=f, app=app, options=user_options)
+
+        return f(ctx, *args, **kwargs)
+
+    return update_wrapper(caller, f)
+
+
 class CeleryOption(click.Option):
     """Customized option for Celery."""
 
