@@ -1,13 +1,11 @@
-# -*- coding: utf-8 -*-
 """Pool implementation abstract factory, and alias definitions."""
-from __future__ import absolute_import, unicode_literals
 
 # Import from kombu directly as it's used
 # early in the import stage, where celery.utils loads
 # too much (e.g., for eventlet patching)
 from kombu.utils.imports import symbol_by_name
 
-__all__ = ('get_implementation',)
+__all__ = ('get_implementation', 'get_available_pool_names',)
 
 ALIASES = {
     'prefork': 'celery.concurrency.prefork:TaskPool',
@@ -28,3 +26,7 @@ else:
 def get_implementation(cls):
     """Return pool implementation by name."""
     return symbol_by_name(cls, ALIASES)
+
+
+def get_available_pool_names():
+    return tuple(ALIASES.keys())
