@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Celery error types.
 
 Error Hierarchy
@@ -45,26 +44,26 @@ Error Hierarchy
         - :class:`~celery.exceptions.DuplicateNodenameWarning`
         - :class:`~celery.exceptions.FixupWarning`
         - :class:`~celery.exceptions.NotConfigured`
+        - :class:`~celery.exceptions.SecurityWarning`
 - :exc:`BaseException`
     - :exc:`SystemExit`
         - :exc:`~celery.exceptions.WorkerTerminate`
         - :exc:`~celery.exceptions.WorkerShutdown`
 """
-from __future__ import absolute_import, unicode_literals
 
 import numbers
 
 from billiard.exceptions import (SoftTimeLimitExceeded, Terminated,
                                  TimeLimitExceeded, WorkerLostError)
+from click import ClickException
 from kombu.exceptions import OperationalError
 
-from .five import python_2_unicode_compatible, string_t
-
 __all__ = (
+    'reraise',
     # Warnings
     'CeleryWarning',
     'AlwaysEagerIgnored', 'DuplicateNodenameWarning',
-    'FixupWarning', 'NotConfigured',
+    'FixupWarning', 'NotConfigured', 'SecurityWarning',
 
     # Core errors
     'CeleryError',
@@ -94,6 +93,8 @@ __all__ = (
 
     # Worker shutdown semi-predicates (inherits from SystemExit).
     'WorkerShutdown', 'WorkerTerminate',
+
+    'CeleryCommandException',
 )
 
 UNREGISTERED_FMT = """\
@@ -101,6 +102,13 @@ Task of kind {0} never registered, please make sure it's imported.\
 """
 
 
+def reraise(tp, value, tb=None):
+    """Reraise exception."""
+    if value.__traceback__ is not tb:
+        raise value.with_traceback(tb)
+    raise value
+
+
 class CeleryWarning(UserWarning):
     """Base class for all Celery warnings."""
 
@@ -121,6 +129,10 @@ class NotConfigured(CeleryWarning):
     """Celery hasn't been configured, as no config module has been found."""
 
 
+class SecurityWarning(CeleryWarning):
+    """Potential security issue found."""
+
+
 class CeleryError(Exception):
     """Base class for all Celery errors."""
 
@@ -129,7 +141,6 @@ class TaskPredicate(CeleryError):
     """Base class for task-related semi-predicates."""
 
 
-@python_2_unicode_compatible
 class Retry(TaskPredicate):
     """The task is to be retried later."""
 
@@ -143,32 +154,33 @@ class Retry(TaskPredicate):
     #: :class:`~datetime.datetime`.
     when = None
 
-    def __init__(self, message=None, exc=None, when=None, is_eager=False, sig=None, **kwargs):
+    def __init__(self, message=None, exc=None, when=None, is_eager=False,
+                 sig=None, **kwargs):
         from kombu.utils.encoding import safe_repr
         self.message = message
-        if isinstance(exc, string_t):
+        if isinstance(exc, str):
             self.exc, self.excs = None, exc
         else:
             self.exc, self.excs = exc, safe_repr(exc) if exc else None
         self.when = when
         self.is_eager = is_eager
         self.sig = sig
-        super(Retry, self).__init__(self, exc, when, **kwargs)
+        super().__init__(self, exc, when, **kwargs)
 
     def humanize(self):
         if isinstance(self.when, numbers.Number):
-            return 'in {0.when}s'.format(self)
-        return 'at {0.when}'.format(self)
+            return f'in {self.when}s'
+        return f'at {self.when}'
 
     def __str__(self):
         if self.message:
             return self.message
         if self.excs:
-            return 'Retry {0}: {1}'.format(self.humanize(), self.excs)
-        return 'Retry {0}'.format(self.humanize())
+            return f'Retry {self.humanize()}: {self.excs}'
+        return f'Retry {self.humanize()}'
 
     def __reduce__(self):
-        return self.__class__, (self.message, self.excs, self.when)
+        return self.__class__, (self.message, self.exc, self.when)
 
 
 RetryTaskError = Retry  # noqa: E305 XXX compat
@@ -178,17 +190,16 @@ class Ignore(TaskPredicate):
     """A task can raise this to ignore doing state updates."""
 
 
-@python_2_unicode_compatible
 class Reject(TaskPredicate):
     """A task can raise this if it wants to reject/re-queue the message."""
 
     def __init__(self, reason=None, requeue=False):
         self.reason = reason
         self.requeue = requeue
-        super(Reject, self).__init__(reason, requeue)
+        super().__init__(reason, requeue)
 
     def __repr__(self):
-        return 'reject requeue=%s: %s' % (self.requeue, self.reason)
+        return f'reject requeue={self.requeue}: {self.reason}'
 
 
 class ImproperlyConfigured(CeleryError):
@@ -211,7 +222,6 @@ class IncompleteStream(TaskError):
     """Found the end of a stream of data, but the data isn't complete."""
 
 
-@python_2_unicode_compatible
 class NotRegistered(KeyError, TaskError):
     """The task is not registered."""
 
@@ -234,7 +244,7 @@ class MaxRetriesExceededError(TaskError):
     def __init__(self, *args, **kwargs):
         self.task_args = kwargs.pop("task_args", [])
         self.task_kwargs = kwargs.pop("task_kwargs", dict())
-        super(MaxRetriesExceededError, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
 
 class TaskRevokedError(TaskError):
@@ -283,7 +293,7 @@ class BackendGetMetaError(BackendError):
 
 
 class BackendStoreError(BackendError):
-    """An issue writing from the backend."""
+    """An issue writing to the backend."""
 
     def __init__(self, *args, **kwargs):
         self.state = kwargs.get('state', "")
@@ -291,3 +301,10 @@ class BackendStoreError(BackendError):
 
     def __repr__(self):
         return super().__repr__() + " state:" + self.state + " task_id:" + self.task_id
+
+
+class CeleryCommandException(ClickException):
+
+    def __init__(self, message, exit_code):
+        super().__init__(message=message)
+        self.exit_code = exit_code
