@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Proxy/PromiseProxy implementation.
 
 This module contains critical utilities that needs to be loaded as
@@ -6,7 +5,6 @@ soon as possible, and that shall not load any third party modules.
 
 Parts of this module is Copyright by Werkzeug Team.
 """
-from __future__ import absolute_import, unicode_literals
 
 import operator
 import sys
@@ -14,8 +12,6 @@ from functools import reduce
 from importlib import import_module
 from types import ModuleType
 
-from .five import PY3, bytes_if_py2, items, string, string_t
-
 __all__ = ('Proxy', 'PromiseProxy', 'try_import', 'maybe_evaluate')
 
 __module__ = __name__  # used by Proxy class body
@@ -36,7 +32,7 @@ def _default_cls_attr(name, type_, cls_value):
     def __get__(self, obj, cls=None):
         return self.__getter(obj) if obj is not None else self
 
-    return type(bytes_if_py2(name), (type_,), {
+    return type(name, (type_,), {
         '__new__': __new__, '__get__': __get__,
     })
 
@@ -52,7 +48,7 @@ def try_import(module, default=None):
         return default
 
 
-class Proxy(object):
+class Proxy:
     """Proxy to another object."""
 
     # Code stolen from werkzeug.local.Proxy.
@@ -111,7 +107,7 @@ class Proxy(object):
             # not sure what this is about
             return getattr(loc, self.__name__)
         except AttributeError:  # pragma: no cover
-            raise RuntimeError('no object bound to {0.__name__}'.format(self))
+            raise RuntimeError(f'no object bound to {self.__name__}')
 
     @property
     def __dict__(self):
@@ -124,7 +120,7 @@ class Proxy(object):
         try:
             obj = self._get_current_object()
         except RuntimeError:  # pragma: no cover
-            return '<{0} unbound>'.format(self.__class__.__name__)
+            return f'<{self.__class__.__name__} unbound>'
         return repr(obj)
 
     def __bool__(self):
@@ -132,6 +128,7 @@ class Proxy(object):
             return bool(self._get_current_object())
         except RuntimeError:  # pragma: no cover
             return False
+
     __nonzero__ = __bool__  # Py2
 
     def __dir__(self):
@@ -289,19 +286,6 @@ class Proxy(object):
     def __reduce__(self):
         return self._get_current_object().__reduce__()
 
-    if not PY3:  # pragma: no cover
-        def __cmp__(self, other):
-            return cmp(self._get_current_object(), other)  # noqa
-
-        def __long__(self):
-            return long(self._get_current_object())  # noqa
-
-        def __unicode__(self):
-            try:
-                return string(self._get_current_object())
-            except RuntimeError:  # pragma: no cover
-                return repr(self)
-
 
 class PromiseProxy(Proxy):
     """Proxy that evaluates object once.
@@ -381,6 +365,7 @@ def maybe_evaluate(obj):
     except AttributeError:
         return obj
 
+
 #  ############# Module Generation ##########################
 
 # Utilities to dynamically
@@ -397,14 +382,11 @@ The module %s is deprecated and will be removed in a future version.
 
 DEFAULT_ATTRS = {'__file__', '__path__', '__doc__', '__all__'}
 
+
 # im_func is no longer available in Py3.
 # instead the unbound method itself can be used.
-if sys.version_info[0] == 3:  # pragma: no cover
-    def fun_of_method(method):
-        return method
-else:
-    def fun_of_method(method):  # noqa
-        return method.im_func
+def fun_of_method(method):
+    return method
 
 
 def getappattr(path):
@@ -465,7 +447,7 @@ COMPAT_MODULES = {
 DEPRECATED_ATTRS = set(COMPAT_MODULES['celery'].keys()) | {'subtask'}
 
 
-class class_property(object):
+class class_property:
 
     def __init__(self, getter=None, setter=None):
         if getter is not None and not isinstance(getter, classmethod):
@@ -506,7 +488,8 @@ class LazyModule(ModuleType):
 
     def __getattr__(self, name):
         if name in self._object_origins:
-            module = __import__(self._object_origins[name], None, None, [name])
+            module = __import__(self._object_origins[name], None, None,
+                                [name])
             for item in self._all_by_module[module.__name__]:
                 setattr(self, item, getattr(module, item))
             return getattr(module, name)
@@ -535,10 +518,10 @@ def create_module(name, attrs, cls_attrs=None, pkg=None,
 
     attrs = {
         attr_name: (prepare_attr(attr) if prepare_attr else attr)
-        for attr_name, attr in items(attrs)
+        for attr_name, attr in attrs.items()
     }
     module = sys.modules[fqdn] = type(
-        bytes_if_py2(modname), (base,), cls_attrs)(bytes_if_py2(name))
+        modname, (base,), cls_attrs)(name)
     module.__dict__.update(attrs)
     return module
 
@@ -556,8 +539,6 @@ def recreate_module(name, compat_modules=None, by_module=None, direct=None,
         operator.add,
         [tuple(v) for v in [compat_modules, origins, direct, attrs]],
     )))
-    if sys.version_info[0] < 3:
-        _all = [s.encode() for s in _all]
     cattrs = {
         '_compat_modules': compat_modules,
         '_all_by_module': by_module, '_direct': direct,
@@ -573,21 +554,21 @@ def recreate_module(name, compat_modules=None, by_module=None, direct=None,
 
 def get_compat_module(pkg, name):
     def prepare(attr):
-        if isinstance(attr, string_t):
+        if isinstance(attr, str):
             return Proxy(getappattr, (attr,))
         return attr
 
     attrs = COMPAT_MODULES[pkg.__name__][name]
-    if isinstance(attrs, string_t):
+    if isinstance(attrs, str):
         fqdn = '.'.join([pkg.__name__, name])
         module = sys.modules[fqdn] = import_module(attrs)
         return module
-    attrs[bytes_if_py2('__all__')] = list(attrs)
+    attrs['__all__'] = list(attrs)
     return create_module(name, dict(attrs), pkg=pkg, prepare_attr=prepare)
 
 
 def get_origins(defs):
     origins = {}
-    for module, attrs in items(defs):
+    for module, attrs in defs.items():
         origins.update({attr: module for attr in attrs})
     return origins
